/* **********************************************************************************
	Hilfsfunktionen.
   ******************************************************************************* */


// Gibt einen integer Wert zurck, closer to 0
function Int(x)
{
	if (x < 0) { 
		return(Math.ceil(x));	// Nchsthhere Ganzzahl (Aufrundung)
	} else 
		return(Math.floor(x));	// Nchstniedrigere Ganzzahl (Abrunden)
}


/* **********************************************************************************
	Berechnung des Julianischen Datums JD und daraus die Rckrechnung des Datums
	und Uhrzeit.
   ******************************************************************************* */

// Ermittelt das Datum aus dem Julianische Datum.
// Nach der Ephemeriden Rechnung von Oliver Montenbruck
// bergeben wird das Julianische Datum und Rckgabe ist
// Tag, Monat, Jahr.
//
function calculateDate(JD)
{
	var day = 0; var month = 0; var year = 0;
	var hour = 0; var minute = 0; var second = 0; var msecond = 0;
	var a = 0; var b = 0; var c = 0; var d = 0; var e = 0; var f = 0;
	var x = 2299161;
	// Korrekturfaktor, da bei der Rckrechnung 1 ms im Ergebnis
	// unterschied ist. Dazu JD + 1 / Millisekunden pro Tag
	JD += 0.0000000115740740740;

	//Datumsberechnung aus dem Ganzzahlenanteil des julianischen Datums
	a = Math.floor(JD + 0.5);
	if ( a < x) { 
		c = a + 1524;
	}else if ( a >= x){
		b = Math.floor( (a - 1867216.25) / 36524.25 );
		c = a + b - Math.floor( b / 4 ) + 1525;
	}
	
	d = Math.floor( ( c - 122.1 ) / 365.25 );
	e = Math.floor ( d * 365.25 );
	f = Math.floor ( ( c - e ) / 30.6001 );
	
	day = Math.trunc(c - e - Math.floor( f * 30.6001 ) + ( JD + 0.5 - a ));
	month = f - 1 - 12 * Math.floor ( f / 14);
	year = d - 4715 - Math.floor ( ( 7 + month ) / 10 );
	
	// Uhrzeit Berechnung aus den Nachkommastellen des Julianischen Datums
	var jdRest = Math.abs( ( ( JD + 0.5 )  - Math.trunc(JD + 0.5) ) );
	hour = Math.abs(Math.trunc(24. * jdRest));
	jdRest = (24. * jdRest) - hour;
	minute = Math.trunc(60. * jdRest);
	jdRest = (60. * jdRest) - minute;
	second = Math.trunc(60. * jdRest);
	jdRest = (60. * jdRest) - second;
	msecond = Math.trunc(1000. * jdRest);

	/*
	var rest = 0;
	var msPerDay = 86400000;
	var msPerHour = 3600000;
	var msPerMinute = 60000;
	var msPerSecond = 1000;
	
	rest = Math.trunc( ( ( JD - Math.trunc(JD) ) - 0.5 ) * msPerDay );
	hour = Math.trunc( rest / msPerHour );
	
	rest = rest - ( hour * msPerHour);
	minute = Math.trunc( rest / msPerMinute );
	
	rest = rest - ( minute * msPerMinute);
	second = Math.trunc( rest / msPerSecond );
	
	rest = rest - ( second * msPerSecond);
	msecond = rest;
	*/
	return [day, month, year, hour, minute, second, msecond];
}



// Ermittelt das Julianische Datum: Nach der Ephemeriden Rechnung.
//
function calculateJD(day,month,year, UT)
{
  // Variablen der Hilfsgren
  var y = 0; var m = 0; var B = 0;
  // Zu berechnendes Datumsobjekt
  var date = new Date(year, month, day);
  // Date Objekte fr die Umstellung von Julianischen- auf
  // Grgorianischen Kalender
  var date1 = new Date("1582", "10", "4");
  var date2 = new Date("1582", "10", "15")
  // Hilfsgren berechnen
  if (month <= 2) {
	y = year - 1;
	m = month + 12; 
  }else{
	y = year;
	m = month;
  }
  /* **************************************************
   * Beschrnkt man sich bei den Berechnungen nur auf den
   * Zeitraum vom 01.03.1900 - 28.02.2100, dann kann man
   * B = -15 setzen.
   * **************************************************/
  // bergebenes Datum liegt im Julianischen Bereich
  if (date <= date1){
	B = -2;
  // bergebenes Datum liegt im Gregorianischen Bereich
  }else if (date >= date2){
	B = Math.floor(y/400) - Math.floor(y/100);
  }
  // UT = GMT, also Lokalzeit - DST (Daylight Savings Time) - Zeitzone
  return (Math.floor(365.25 * y) + Math.floor(30.6001 * (m + 1)) + B + 1720996.5 + day + (UT/24));

}


// Ermittelt das Modified Julianische Datum
// aus dem Julianischen Datums ab 1900.
//
function calculateMJD(JD)
{
	// Korrekturfaktor, da bei der Rckrechnung 1 ms im Ergebnis
	// unterschied ist. Dazu JD + 1 / Millisekunden pro Tag
	JD += 0.0000000115740740740;

	return (JD - 2400000.5);
}

// Ermittelt das Modified Julianische Datum
// aus dem Julianischen Datums.
//
function calculateDateFromMJD(MJD)
{
	var day = 0; var month = 0; var year = 0;
	var hour = 0; var minute = 0; var second = 0; var msecond = 0;
	var K = 0;
	
	year = Int ( (MJD - 15078.2) / 365.25 );
	month = Int ( ( MJD - 14956.1 - Int (year * 365.25) ) / 30.6001 );
	day = Math.trunc(MJD - 14956 - Int ( year * 365.25 ) - Int ( month * 30.6001 ));

	if ( (month == 14) || (month == 15) )
		K = 1;
	else 
		K = 0;
		
	year += (K + 1900);
	month = month - 1 - (K * 12);

	// Uhrzeit Berechnung aus den Nachkommastellen des Julianischen Datums
	var jdRest = Math.abs( ( ( MJD )  - Math.trunc(MJD) ) );
	hour = Math.abs(Math.trunc(24. * jdRest));
	jdRest = (24. * jdRest) - hour;
	minute = Math.trunc(60. * jdRest);
	jdRest = (60. * jdRest) - minute;
	second = Math.trunc(60. * jdRest);
	jdRest = (60. * jdRest) - second;
	msecond = Math.trunc(1000. * jdRest);

  return [day, month, year, hour, minute, second, msecond];
}